@@ -34,6 +34,8 @@ import javax.accessibility.AccessibleContext;
 import javax.accessibility.AccessibleRole;
 import javax.accessibility.AccessibleText;
 import javax.accessibility.AccessibleEditableText;
+import sun.misc.Signal;
+import sun.misc.SignalHandler;
 
 public class JavaBridge {
 
@@ -52,8 +54,10 @@ public class JavaBridge {
  static String oldEventName = null;
  static int oldDetail1 = -1;
  static boolean debugFlag = false;
+ static Application the_app;
 
  javax.accessibility.Accessible appAccessible;
+ SignalHandler h = Signal.handle (new Signal("PIPE"), SignalHandler.SIG_IGN);
 
 	static {
 	 accessibleObjectFactory = new AccessibleFactory();
@@ -87,7 +91,15 @@ public class JavaBridge {
 				}
 				if (ac != null) {
 					org.GNOME.Accessibility.Accessible accessible = getAccessibleObjectFactory().getAccessible(ac);
-					any.insert_Object (accessible, AccessUtil.getORB().get_primitive_tc (TCKind.tk_objref));
+
+					EventDetails ed = new EventDetails();
+					ed.host_application = the_app;
+					ed.source_role = accessible.getRole();
+					ed.source_name = accessible.name();
+					ed.any_data = AccessUtil.getORB().create_any ();
+					AccessibleHelper.insert(ed.any_data, accessible);
+					EventDetailsHelper.insert (any, ed);
+
 					detail1 = ac.getAccessibleIndexInParent ();
 				}
 				dispatchEvent(source, 
@@ -121,7 +133,7 @@ public class JavaBridge {
 				if ( textEvent ) {
 				    int selection_start = 0, selection_end = 0;
 				    int selection_change_start = 0, selection_change_end = 0;
-				    org.omg.CORBA.Any selection_any = AccessUtil.getORB().create_any ();
+				    org.omg.CORBA.Any any = AccessUtil.getORB().create_any ();
 				    AccessibleContext ac = null;
 				    if (source instanceof AccessibleContext) 
 					ac = (AccessibleContext) source;
@@ -162,14 +174,21 @@ public class JavaBridge {
 					selection_change_end = selection_end;
 				    }
 				    if (new_selection) {
-					selection_any.insert_string (oldFocusText.substring (
+					 org.GNOME.Accessibility.Accessible accessible = getAccessibleObjectFactory().getAccessible(ac);
+					EventDetails ed = new EventDetails();
+					ed.host_application = the_app;
+					ed.source_role =  accessible.getRole();
+					ed.source_name = accessible.name();
+					ed.any_data = AccessUtil.getORB().create_any();
+					ed.any_data.insert_string (oldFocusText.substring (
 									 Math.min (selection_change_start, 
 									      selection_change_end),
 									 Math.max (selection_change_start,
 									      selection_change_end)));
+					EventDetailsHelper.insert (any, ed);
 					dispatchEvent(source,
 						      "object:text-selection-changed", 
-						      selection_start, selection_end, selection_any);
+						      selection_start, selection_end, any);
 				    }
 				}
 				else {
@@ -331,6 +350,18 @@ public class JavaBridge {
 		if (debugFlag)
 			System.err.println ("Java Accessibility Bridge for GNOME loaded.\n");
 
+        try {
+            String javaHome = System.getProperty ("java.home");
+            System.load ( javaHome + "/lib/ext/libjava-access-bridge-jni.so" );
+			if (debugFlag)
+				System.err.println ( javaHome + "/lib/ext/libjava-access-bridge-jni.so loaded" );
+        }
+        catch (Exception ex)
+        {
+            System.err.println(ex+" caught.");
+            ex.printStackTrace();
+        }
+
 		// Not sure what kind of arguments should be sent to ORB
 		String vm_rev = System.getProperty("java.version");
 
@@ -408,9 +439,20 @@ public class JavaBridge {
 //                                    eventSource.toString() );
 				return;
 			}
+
 			org.GNOME.Accessibility.Accessible accessible = 
 				getAccessibleObjectFactory().
 				getAccessible(ac);
+
+			if (!any.type().equivalent(EventDetailsHelper.type())) {
+				EventDetails ed = new EventDetails();
+				ed.host_application = the_app;
+				ed.source_role = accessible.getRole();
+				ed.source_name = accessible.name();
+				ed.any_data = AccessUtil.getORB().create_any();
+				EventDetailsHelper.insert (any, ed);
+			}
+
 			org.GNOME.Accessibility.Event event = 
 				new org.GNOME.Accessibility.Event (eventName, 
 								   accessible,
@@ -453,7 +495,6 @@ public class JavaBridge {
 
 	public static void dispatchFocusEvent(java.lang.Object eventSource) {
 		AccessibleContext ctx;
-		org.omg.CORBA.Any any = AccessUtil.getORB().create_any ();
 
 		// System.err.println ("dispatchFocusEvent: " + eventSource);
 		if (eventSource == null) {
@@ -546,6 +587,14 @@ public class JavaBridge {
             }
   			org.GNOME.Accessibility.Accessible accessible = 
 			      JavaBridge.getAccessibleObjectFactory().getAccessible(ctx);
+
+			org.omg.CORBA.Any any = AccessUtil.getORB().create_any ();
+			EventDetails ed = new EventDetails();
+			ed.host_application = the_app;
+			ed.source_role = accessible.getRole();
+			ed.source_name = accessible.name();
+			ed.any_data =  AccessUtil.getORB().create_any ();
+			EventDetailsHelper.insert (any, ed);
 			org.GNOME.Accessibility.Event event = new org.GNOME.Accessibility.Event
 				("focus:", accessible, 0, 0, any);
 			// System.out.println("focus: " + ctx);
@@ -589,15 +638,20 @@ public class JavaBridge {
 			}
 			oldFocusSelectStart = selection_start;
 			oldFocusSelectEnd = selection_end;
-			if (selection_change_start != selection_change_end) {
-			    selection_any.insert_string (oldFocusText.substring (
-							     Math.min (selection_change_start, selection_change_end),
-							     Math.max (selection_change_start, selection_change_end)));
-			    dispatchEvent (e.getSource(),
-					   "object:text-selection-changed",
-					   selection_start, selection_end,
-					   selection_any);
-			}
+			try {
+				if (selection_change_start != selection_change_end) {
+				    selection_any.insert_string (oldFocusText.substring (
+								     Math.min (selection_change_start, selection_change_end),
+								     Math.max (selection_change_start, selection_change_end)));
+				    dispatchEvent (e.getSource(),
+						   "object:text-selection-changed",
+						   selection_start, selection_end,
+						   selection_any);
+				}
+		    } catch(StringIndexOutOfBoundsException indexException) { 
+		    	// the selection_change_end index may be more than the 
+		    	// length of the string. ignore this case silently
+		    }
 		    }
 		}
 		dispatchEvent(e.getSource(), 
@@ -711,11 +765,16 @@ public class JavaBridge {
 			    oldFocusText = aeText.getTextRange (0, charCount);
 		    }
 		    else if ((ac != null) && (ac == oldFocusSource)) {
+			// accessibleAWTTextComponent
+			// the event fired by the text component is different from what we expect
+			// newValueObject is not the position of the insertion, 
+			// but the position of the caret after the instertion, so fix newValueObject
+			newValueObject = new Integer(((Integer)newValueObject).intValue() - oldValue);
+			oldFocusText = getTextRange (aText, 0, charCount);
 			any.insert_string (oldFocusText.substring (
 				       ((Integer)newValueObject).intValue(),
 				       ((Integer)newValueObject).intValue() + 
 				       oldValue));
-			oldFocusText = getTextRange (aText, 0, charCount);
 		    }
 		}
                 else if ( charCount < prevCharCount ) {
@@ -1064,6 +1123,7 @@ public class JavaBridge {
             
 			Application aref = getAccessibleObjectFactory().
                                getApplication(appAccessible.getAccessibleContext());
+			the_app = aref;
             
 			if (debugFlag)
 				System.err.println(aref);
@@ -1154,6 +1214,8 @@ public class JavaBridge {
 				isConsumed = JavaBridge.dispatchDeviceEvent (de);
 				if ( isConsumed ) pendingKeyPress = null;
 			} else if (e.getID () == KeyEvent.KEY_RELEASED) {
+				if (pendingKeyPress != null) 
+					super.dispatchEvent (pendingKeyPress);
 				de = KeyEventAdapter.deviceEventFromKeyEvent (e);
 				isConsumed = JavaBridge.dispatchDeviceEvent (de);
 				pendingKeyPress = null;
